@@ -1,5 +1,5 @@
 /* Compress or decompress an ELF file.
-   Copyright (C) 2015 Red Hat, Inc.
+   Copyright (C) 2015, 2016 Red Hat, Inc.
    This file is part of elfutils.
 
    This file is free software; you can redistribute it and/or modify
@@ -32,7 +32,9 @@
 #include <unistd.h>
 #include ELFUTILS_HEADER(elf)
 #include ELFUTILS_HEADER(ebl)
+#include ELFUTILS_HEADER(dwelf)
 #include <gelf.h>
+#include "libeu.h"
 #include "system.h"
 
 /* Name and version of program.  */
@@ -101,6 +103,7 @@ parse_opt (int key, char *arg __attribute__ ((unused)),
 
     case 'q':
       verbose--;
+      break;
 
     case 'f':
       force = true;
@@ -264,9 +267,9 @@ process_file (const char *fname)
   char *snamebuf = NULL;
 
   /* String table (and symbol table), if section names need adjusting.  */
-  struct Ebl_Strtab *names = NULL;
-  struct Ebl_Strent **scnstrents = NULL;
-  struct Ebl_Strent **symstrents = NULL;
+  Dwelf_Strtab *names = NULL;
+  Dwelf_Strent **scnstrents = NULL;
+  Dwelf_Strent **symstrents = NULL;
   char **scnnames = NULL;
 
   /* Section data from names.  */
@@ -307,7 +310,7 @@ process_file (const char *fname)
     free (snamebuf);
     if (names != NULL)
       {
-	ebl_strtabfree (names);
+	dwelf_strtab_free (names);
 	free (scnstrents);
 	free (symstrents);
 	free (namesbuf);
@@ -523,14 +526,14 @@ process_file (const char *fname)
 
   if (adjust_names)
     {
-      names = ebl_strtabinit (true);
+      names = dwelf_strtab_init (true);
       if (names == NULL)
 	{
 	  error (0, 0, "Not enough memory for new strtab");
 	  return cleanup (-1);
 	}
       scnstrents = xmalloc (shnum
-			    * sizeof (struct Ebl_Strent *));
+			    * sizeof (Dwelf_Strent *));
       scnnames = xcalloc (shnum, sizeof (char *));
     }
 
@@ -869,7 +872,7 @@ process_file (const char *fname)
 
 	  /* We need to keep a copy of the name till the strtab is done.  */
 	  name = scnnames[ndx] = xstrdup (name);
-	  if ((scnstrents[ndx] = ebl_strtabadd (names, name, 0)) == NULL)
+	  if ((scnstrents[ndx] = dwelf_strtab_add (names, name)) == NULL)
 	    {
 	      error (0, 0, "No memory to add section name string table");
 	      return cleanup (-1);
@@ -915,7 +918,7 @@ process_file (const char *fname)
 		}
 	      size_t elsize = gelf_fsize (elfnew, ELF_T_SYM, 1, EV_CURRENT);
 	      size_t syms = symd->d_size / elsize;
-	      symstrents = xmalloc (syms * sizeof (struct Ebl_Strent *));
+	      symstrents = xmalloc (syms * sizeof (Dwelf_Strent *));
 	      for (size_t i = 0; i < syms; i++)
 		{
 		  GElf_Sym sym_mem;
@@ -937,7 +940,7 @@ process_file (const char *fname)
 			  error (0, 0, "Couldn't get symbol %zd name", i);
 			  return cleanup (-1);
 			}
-		      symstrents[i] = ebl_strtabadd (names, symname, 0);
+		      symstrents[i] = dwelf_strtab_add (names, symname);
 		      if (symstrents[i] == NULL)
 			{
 			  error (0, 0, "No memory to add to symbol name");
@@ -969,7 +972,11 @@ process_file (const char *fname)
 	  error (0, 0, "Couldn't create new section header string table data");
 	  return cleanup (-1);
 	}
-      ebl_strtabfinalize (names, data);
+      if (dwelf_strtab_finalize (names, data) == NULL)
+	{
+	  error (0, 0, "Not enough memory to create string table");
+	  return cleanup (-1);
+	}
       namesbuf = data->d_buf;
 
       GElf_Shdr shdr_mem;
@@ -983,7 +990,7 @@ process_file (const char *fname)
 
       /* Note that we also might have to compress and possibly set
 	 sh_off below */
-      shdr->sh_name = ebl_strtaboffset (scnstrents[shdrstrndx]);
+      shdr->sh_name = dwelf_strent_off (scnstrents[shdrstrndx]);
       shdr->sh_type = SHT_STRTAB;
       shdr->sh_flags = 0;
       shdr->sh_addr = 0;
@@ -1098,7 +1105,7 @@ process_file (const char *fname)
 	    }
 
 	  if (adjust_names)
-	    shdr->sh_name = ebl_strtaboffset (scnstrents[ndx]);
+	    shdr->sh_name = dwelf_strent_off (scnstrents[ndx]);
 
 	  if (gelf_update_shdr (scn, shdr) == 0)
 	    {
@@ -1132,7 +1139,7 @@ process_file (const char *fname)
 
 		  if (sym->st_name != 0)
 		    {
-		      sym->st_name = ebl_strtaboffset (symstrents[i]);
+		      sym->st_name = dwelf_strent_off (symstrents[i]);
 
 		      if (gelf_update_sym (symd, i, sym) == 0)
 			{
